/*******************************************************************************
* File Name: Debounce_Timer_PM.c  
* Version 2.0
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "Debounce_Timer.h"
static Debounce_Timer_backupStruct Debounce_Timer_backup;

/*******************************************************************************
* Function Name: Debounce_Timer_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  Debounce_Timer_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
* Reentrant:
*    No
*
*******************************************************************************/
void Debounce_Timer_SaveConfig(void)
{   
    #if (!Debounce_Timer_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (Debounce_Timer_PSOC3_ES2 || Debounce_Timer_PSOC5_ES1)
            Debounce_Timer_backup.TimerUdb = Debounce_Timer_ReadCounter();
            Debounce_Timer_backup.TimerPeriod = Debounce_Timer_ReadPeriod();
            Debounce_Timer_backup.InterruptMaskValue = Debounce_Timer_STATUS_MASK;
            #if (Debounce_Timer_UsingHWCaptureCounter)
                Debounce_Timer_backup.TimerCaptureCounter = Debounce_Timer_ReadCaptureCount();        
            #endif
        #endif
        
        #if (Debounce_Timer_PSOC3_ES3 || Debounce_Timer_PSOC5_ES2)
            Debounce_Timer_backup.TimerUdb = Debounce_Timer_ReadCounter();
            Debounce_Timer_backup.InterruptMaskValue = Debounce_Timer_STATUS_MASK;
            #if (Debounce_Timer_UsingHWCaptureCounter)
                Debounce_Timer_backup.TimerCaptureCounter = Debounce_Timer_ReadCaptureCount();        
            #endif
        #endif
        
        #if(!Debounce_Timer_ControlRegRemoved)
            Debounce_Timer_backup.TimerControlRegister = Debounce_Timer_ReadControlRegister();
        #endif
    #endif
}

/*******************************************************************************
* Function Name: Debounce_Timer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Debounce_Timer_backup:  Variables of this global structure are used to restore 
*  the values of non retention registers on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void Debounce_Timer_RestoreConfig(void) 
{
    #if (!Debounce_Timer_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (Debounce_Timer_PSOC3_ES2 || Debounce_Timer_PSOC5_ES1)
            Debounce_Timer_WriteCounter(Debounce_Timer_backup.TimerUdb);
            Debounce_Timer_WritePeriod(Debounce_Timer_backup.TimerPeriod);
            Debounce_Timer_STATUS_MASK =Debounce_Timer_backup.InterruptMaskValue;
            #if (Debounce_Timer_UsingHWCaptureCounter)
                Debounce_Timer_SetCaptureCount(Debounce_Timer_backup.TimerCaptureCounter);        
            #endif
        #endif
        
        #if (Debounce_Timer_PSOC3_ES3 || Debounce_Timer_PSOC5_ES2)
            Debounce_Timer_WriteCounter(Debounce_Timer_backup.TimerUdb);
            Debounce_Timer_STATUS_MASK =Debounce_Timer_backup.InterruptMaskValue;
            #if (Debounce_Timer_UsingHWCaptureCounter)
                Debounce_Timer_SetCaptureCount(Debounce_Timer_backup.TimerCaptureCounter);           
            #endif
        #endif
        
        #if(!Debounce_Timer_ControlRegRemoved)
            Debounce_Timer_WriteControlRegister(Debounce_Timer_backup.TimerControlRegister);
        #endif
    #endif
}


/*******************************************************************************
* Function Name: Debounce_Timer_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Debounce_Timer_backup.TimerEnableState:  Is modified depending on the enable state
*  of the block before entering sleep mode.
*
* Reentrant:
*    No
*
*******************************************************************************/
void Debounce_Timer_Sleep(void)
{
    #if(!Debounce_Timer_ControlRegRemoved)
        /* Save Counter's enable state */
        if(Debounce_Timer_CTRL_ENABLE == (Debounce_Timer_CONTROL & Debounce_Timer_CTRL_ENABLE))
        {
            /* Timer is enabled */
            Debounce_Timer_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            Debounce_Timer_backup.TimerEnableState = 0u;
        }
    #endif
    Debounce_Timer_Stop();
    Debounce_Timer_SaveConfig();
}


/*******************************************************************************
* Function Name: Debounce_Timer_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Debounce_Timer_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void Debounce_Timer_Wakeup(void) 
{
    Debounce_Timer_RestoreConfig();
    #if(!Debounce_Timer_ControlRegRemoved)
        if(Debounce_Timer_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                Debounce_Timer_Enable();
        } /* Do nothing if Timer was disabled before */ 
    #endif
}


/* [] END OF FILE */
